Package: GitStats
Title: Standardized Git Repository Data
Version: 2.5.2
Authors@R: c(
    person(given = "Maciej", family = "Banas", email = "banasmaciek@gmail.com", role = c("aut", "cre")),
    person(given = "Kamil", family = "Koziej", email = "koziej.k@gmail.com", role = "aut"),
    person(given = "Karolina", family = "Marcinkowska", email = "karolina_marcinkowska@onet.pl", role = "aut"),
    person(given = "Matt", family = "Secrest", email = "secrestm@gene.com", role = "aut")
  )
Maintainer: Maciej Banas <banasmaciek@gmail.com>
Description: Obtain standardized data from multiple 'Git' services, including 'GitHub' and 'GitLab'. 
    Designed to be 'Git' service-agnostic, this package assists teams with activities spread across various 
    'Git' platforms by providing a unified way to access repository data.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
URL: https://r-world-devs.github.io/GitStats/,
        https://github.com/r-world-devs/GitStats
BugReports: https://github.com/r-world-devs/GitStats/issues
Imports: cli, dplyr, glue, httr2, lubridate (>= 1.8.0), mirai (>=
        2.0.0), rlang (>= 1.1.0), R6, purrr (>= 1.0.0), stringr
Suggests: DBI, jsonlite, RPostgres, RSQLite, spelling, knitr, mockery,
        rmarkdown, testthat (>= 3.0.0), withr
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
Depends: R (>= 4.1.0)
NeedsCompilation: no
Packaged: 2026-05-05 10:39:52 UTC; banasm
Author: Maciej Banas [aut, cre],
  Kamil Koziej [aut],
  Karolina Marcinkowska [aut],
  Matt Secrest [aut]
Repository: CRAN
Date/Publication: 2026-05-05 13:14:10 UTC
Built: R 4.5.2; ; 2026-05-05 13:44:04 UTC; unix
