# Generated by roxygen2: do not edit by hand

S3method(print,gitstats_repos_urls)
export(create_gitstats)
export(get_commits)
export(get_commits_stats)
export(get_files)
export(get_issues)
export(get_issues_stats)
export(get_orgs)
export(get_pull_requests)
export(get_pull_requests_stats)
export(get_release_logs)
export(get_repos)
export(get_repos_trees)
export(get_repos_urls)
export(get_storage)
export(get_storage_metadata)
export(get_users)
export(is_parallel)
export(remove_from_storage)
export(remove_postgres_storage)
export(remove_sqlite_storage)
export(set_github_host)
export(set_gitlab_host)
export(set_local_storage)
export(set_parallel)
export(set_postgres_storage)
export(set_sqlite_storage)
export(show_hosts)
export(show_orgs)
export(verbose_off)
export(verbose_on)
importFrom(R6,R6Class)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_alert_warning)
importFrom(cli,col_green)
importFrom(cli,col_yellow)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,glimpse)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(glue,glue)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(lubridate,floor_date)
importFrom(mirai,daemons)
importFrom(mirai,everywhere)
importFrom(mirai,mirai_map)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(rlang,"%||%")
importFrom(rlang,expr)
importFrom(stats,runif)
importFrom(stringr,str_length)
importFrom(stringr,str_replace)
