# This is a solution for warnings in r-cmd checks of global variables
globalVariables(c(
  ".", "fullname", "organization", "repo_url",
  "repo_name", "created_at", "last_activity_at", "last_activity", "stats_date",
  "committed_date", "commits_n", "api_url", "row_no", ".N", ".data",
  "repository", "stars", "forks", "languages", "issues_open", "issues_closed",
  "contributors_n", "githost", "head", "state", "merged_at"
))

cli_icons <- list(
  org = "\U1F3DB\UFE0F",
  repo = "\U1F310",
  commit = "\U1F552",
  issue = "\U1F41B",
  language = "\U1F3AD",
  pull_request = "\U1F500",
  user = "\U1F64D",
  file = "\U1F4C4",
  tree = "\U1F333",
  release = "\U1F3F7\UFE0F"
)

text_ext_files <- "\\.(txt|md|qmd|Rmd|markdown|yaml|yml|csv|json|xml|html|htm|css|js|R|r|py|sh|bat|ini|conf|log|sql|tsv|mdx)$"
no_ext_files <- "^[^\\.]+$"

text_files_pattern <- paste0("(", text_ext_files, "|", no_ext_files, ")")
