# `get_commits_authors_handles_and_names()` prints messages

    Code
      gl_commits_table <- test_rest_gitlab$get_commits_authors_handles_and_names(
        commits_table = test_mocker$use("gl_commits_table"), verbose = TRUE)
    Message
      > [Host:GitLab][Engine:REST] Looking up for authors' 🙍 names and logins...

# get_commits_from_orgs works

    Code
      gl_commits_table <- gitlab_testhost_priv$get_commits_from_orgs(since = "2023-03-01",
        until = "2023-04-01", verbose = TRUE, progress = FALSE)
    Message
      > [mbtests] Pulling repositories 🌐 data...
      > Caching repositories for [mbtests]...
      > [Host:GitLab][Engine:REST][Scope:mbtests] Pulling commits 🕒...

# get_commits_from_repos works

    Code
      gl_commits_table <- gitlab_testhost_priv$get_commits_from_repos(since = "2023-03-01",
        until = "2023-04-01", verbose = TRUE, progress = FALSE)
    Message
      > [Host:GitLab][Engine:REST][Scope:test_org: 1 repos] Pulling commits 🕒...

