# get_pull_requests_stats returns error when issues is not pr_data object

    x `pull_requests` must be a `gitstats_issues` object.
    i Pull first your pull_requests with `get_pull_requests()` function.

