# get_repos returns warning when repositories table is empty

    Code
      repos_data <- test_gitstats$get_repos(cache = FALSE, verbose = TRUE)
    Message
      i Cache set to FALSE, I will pull data from API.
      > Pulling repositories 🌐 data...
      ! No repositories 🌐 found.

# get_repos prints time used to pull data

    Code
      repos_data <- get_repos(gitstats = test_gitstats, verbose = TRUE)
    Message
      v Data pulled in 0 secs

