% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Storage.R
\name{remove_sqlite_storage}
\alias{remove_sqlite_storage}
\title{Remove SQLite storage}
\usage{
remove_sqlite_storage(gitstats)
}
\arguments{
\item{gitstats}{A GitStats object.}
}
\value{
A \code{GitStats} object (invisibly).
}
\description{
Closes the SQLite connection and, for file-based databases,
deletes the database file. For in-memory databases the data is simply
discarded. Storage is reverted to the default in-memory local backend.
Errors if no SQLite backend is currently set.
}
\examples{
\dontrun{
  my_gitstats <- create_gitstats() |>
    set_sqlite_storage(dbname = "gitstats.sqlite")
  remove_sqlite_storage(my_gitstats)
}
}
