% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GitStats.R
\name{is_verbose}
\alias{is_verbose}
\title{Is verbose mode switched on}
\usage{
is_verbose(gitstats)
}
\arguments{
\item{gitstats}{A GitStats object.}
}
\description{
Is verbose mode switched on
}
