% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Storage.R
\name{get_storage_metadata}
\alias{get_storage_metadata}
\title{Get metadata for a storage table}
\usage{
get_storage_metadata(gitstats, storage = NULL)
}
\arguments{
\item{gitstats}{A GitStats object.}

\item{storage}{A character, name of the table (e.g. \code{"commits"},
\code{"repositories"}). If \code{NULL} (default), metadata for all tables in storage
will be returned.}
}
\value{
A list with metadata fields: \code{class}, \code{attributes}, and (for
database backends) \code{column_types}.
}
\description{
Retrieves metadata (R classes, custom attributes, column types)
for a table stored in the active storage backend.
}
\examples{
\dontrun{
  my_gitstats <- create_gitstats() |>
    set_github_host(
      token = Sys.getenv("GITHUB_PAT"),
      orgs = "r-world-devs"
    )
  get_commits(my_gitstats, since = "2024-01-01")
  get_storage_metadata(my_gitstats, storage = "commits")
}
}
